STORE_NAME = '.kaboom.json'
HELP = '''
Usage ---  kaboom [-h] action [...]
  kaboom: help -------------------------------------------------------

  kaboom                        this help text
  kaboom help                   this help text

  kaboom add <name> <value>     Add name:value to kaboom

  kaboom show <name>            display value of name
  kaboom show all               display everything stored in kaboom

  kaboom delete <name>          Remove key <name>
  kaboom delete all             Remove all keys

  kaboom update <name> <value>  Updates name's value to supplied value

  all other documentation is located at:
    https://github.com/vinu76jsr/kaboom
  Inspired by https://github.com/holman/boom'''


from distutils.core import setup
from kaboom import __version__

import os

description = 'Command line key-value store'
long_description = description
if os.path.exists('README.rst'):
    long_description = open('README.rst').read()


setup(
    name='kaboom',
    version=__version__,
    packages=['kaboom'],
    url='http://github.com/vinu76jsr/kaboom',
    author='vaibhav',
    author_email='vinu76jsr@gmail.com',
    license='bsd3',
    description=description,
    entry_points={'console_scripts': [
        'kaboom = kaboom.kaboom:run']},
    long_description=long_description
)



import os
import sys
import json
from os.path import expanduser
from constants import HELP, STORE_NAME


home_dir = expanduser('~')


def _exit(message):
    if Store.file_obj:
        Store.file_obj.close()
    sys.exit(message)


class Command(object):

    def __init__(self, name, group=None):
        self.name = name
        self.group = group


class Group(object):

    def __init__(self, name, commands=None):
        self.name = name
        self.commands = commands

    @property
    def count(self):
        return len(self.commands)


class Store(object):

    file_obj = None
    data = None
    _path = os.path.join(home_dir, STORE_NAME)

    @classmethod
    def init_data(cls):
        if os.path.exists(cls._path):
            with open(cls._path, 'r') as file_obj:
                _data = file_obj.read()
                cls.data = json.loads(_data)  # data is loaded
            if cls.data is None:
                _exit('Store load failed with')
        else:
            with open(cls._path, 'w+') as _:  # create the file
                cls.data = {}
                _.write(json.dumps(cls.data))


    @classmethod
    def close(cls):
        cls.save()

    @classmethod
    def save(cls):
        if cls.file_obj:
            cls.close()

        if Store.data is not None:
            with open(cls._path, 'w+') as file_obj:
                file_obj.write(json.dumps(Store.data))


class App(object):

    HELP = 'help'
    SHOW = 'show'
    ADD = 'add'
    DELETE = 'delete'
    UPDATE = 'update'

    SUPPORTED_COMMANDS = (HELP, SHOW, ADD, DELETE, UPDATE)

    def __init__(self):
        Store.init_data()
        if Store.data is None:
            _exit('Cannot initialize data')
        # Now we have store taken care of start working

    def call(self, command, *arguments):
        if command not in self.SUPPORTED_COMMANDS:
            print ("Command `%s` is not supported, "
                   "try one of following commands - %s " % (command,
                                                            ', '.join(self.SUPPORTED_COMMANDS)))
            return
        try:
            attr = getattr(self, command)
        except AttributeError:
            print ("Command proc is not supported, "
                   "try one of following commands %s " % str(self.SUPPORTED_COMMANDS))
            return
        attr(*arguments)
        Store.close()

    @staticmethod
    def show(*args):
        if not args:
            _exit('What to show?')
        # print 'show called with args %s' % args
        if len(args) > 1:
            _exit('Too many arguments')
        name = args[0]
        if name == 'all':
            if len(Store.data):
                print "Showing all items"
                for name, value in Store.data.iteritems():
                    print '%s : %s' % (name, value)
                return
            else:
                print "Nothing to show"
                return
        # print 'show called with %s arguments:%s' % (len(args), args)
        if name in Store.data:
            print Store.data[name]
        else:
            _exit('%s not present' % args[0])

    @staticmethod
    def add(*args):
        if not args:
            print 'adding Nothing'
            return
        # check if same name is present as a command
        name = args[0]
        if len(args) < 2:
            print 'provide value for %s' % name
            return
        value = ' '.join(args[1:])
        # add as a command
        if name not in Store.data:
            Store.data[name] = value
            print "Value %s added for item %s" % (value, name)
        else:
            print "already present, use update(Not supported yet) if you want to change it"

    @staticmethod
    def delete(*args):
        if not args:
            print 'Pass all to delete everything'
            return
        if args[0] == 'all':
            items_to_be_deleted = len(Store.data)
            Store.data = {}
            print 'Deleted everything(%s items)!' % items_to_be_deleted
            return
        name = args[0]

        Store.data.pop(name, None)
        print "Deleted %s" % name

    def update(self, *args):
        if not args:
            print 'update what?'
            return
        if len(args) < 2:
            print "require at least 2 arguments"
            return
        name = args[0]
        value = ' '.join(args[1:])

        if name not in Store.data:
            print "%s not in Store, adding" % name
            self.add(name, value)
            return
        else:
            Store.data[name] = value

    def help(self):
        print HELP
























#!/usr/bin/env python
import argparse
from app import App
from utils import did_you_mean


def run():
    parser = argparse.ArgumentParser(description='Command line key-value store.',
                                     add_help=False, )
    parser.add_argument('action', metavar='action', nargs='*', help='action')
    args = parser.parse_args()
    # command = args or 'help'
    if not args.action:
        command = 'help'
    else:
        command = args.action[0]
    application = App()

    if not hasattr(application, command):
        print "Command `%s` not found, Did you mean `%s`" % (command,
                                                             did_you_mean(command, App.SUPPORTED_COMMANDS))
    else:
        sub_commands = args.action[1:] if args.action else None
        if sub_commands:
            application.call(command, *sub_commands)
        else:
            application.call(command)


if __name__ == "__main__":
    run()



 # following description appears in levenshtein distance offered by git
 # at https://github.com/git/git/blob/master/levenshtein.c

 # This function implements the Damerau-Levenshtein algorithm to
 # calculate a distance between strings.
 #
 # Basically, it says how many letters need to be swapped, substituted,
 # deleted from, or added to string1, at least, to get string2.
 #
 # The idea is to build a distance matrix for the substrings of both
 # strings.  To avoid a large space complexity, only the last three rows
 # are kept in memory (if swaps had the same or higher cost as one deletion
 # plus one insertion, only two rows would be needed).
 #
 # At any stage, "i + 1" denotes the length of the current substring of
 # string1 that the distance is calculated for.
 #
 # row2 holds the current row, row1 the previous row (i.e. for the substring
 # of string1 of length "i"), and row0 the row before that.
 #
 # In other words, at the start of the big loop, row2[j + 1] contains the
 # Damerau-Levenshtein distance between the substring of string1 of length
 # "i" and the substring of string2 of length "j + 1".
 #
 # All the big loop does is determine the partial minimum-cost paths.
 #
 # It does so by calculating the costs of the path ending in characters
 # i (in string1) and j (in string2), respectively, given that the last
 # operation is a substitution, a swap, a deletion, or an insertion.
 #
 # This implementation allows the costs to be weighted:
 #
 # - w (as in "sWap")
 # - s (as in "Substitution")
 # - a (for insertion, AKA "Add")
 # - d (as in "Deletion")
 #
 #  Note that this algorithm calculates a distance _iff_ d == a.


def levenshtein(string1, string2, swap=0, substitution=2, insertion=1, deletion=3):
    """
    This is levenshtein distance calculation utility taken from
    https://github.com/git/git/blob/master/levenshtein.c

    @param string1:
    @param string2:
    @param swap:
    @param substitution:
    @param insertion:
    @param deletion:
    @return:
    """
    len1 = len(string1)
    len2 = len(string2)

    row0 = list()
    row1 = list()
    row2 = [0] * (len2 + 1)

    for j in range(len2 + 1):
        row1.append(j * insertion)

    for i in range(len1):
        # print i
        dummy = [0] * (len2 + 1)
        row2[0] = (i + 1) * deletion
        for j in range(len2):
            # substitution
            row2[j + 1] = row1[j] + substitution * (string1[i] != string2[j])
            # swap
            if (i > 0 and j > 0 and string1[i-1] == string2[j]
                    and string1[i] == string2[j-1] and
                    row2[j+1] > row0[j-1] + swap):
                row2[j + 1] = row0[j - 1] + swap
            # deletion
            if row2[j + 1] > row1[j + 1] + deletion:
                row2[j + 1] = row1[j + 1] + deletion
            # insertion
            if row2[j + 1] > row2[j] + insertion:
                row2[j + 1] = row2[j] + insertion
        dummy, row0, row1, row2 = row0, row1, row2, dummy

    return row1[len2]


def did_you_mean(string1, strings):
    """
    Takes string1 and calculates levenshtein distance from strings
    and returns most likely string
    @param string1:
    @param strings:
    """
    if not strings:
        return None
    result = strings[0]
    result_distance = levenshtein(string1, strings[0])
    for command in strings[1:]:
        new_distance = levenshtein(string1, command)
        if new_distance < result_distance:
            result, result_distance = command, new_distance

    return result
